;(function(global, factory) {
  if (typeof define === 'function' && define.amd) {
    define(['exports', 'module'], factory)
  } else if (typeof exports !== 'undefined' && typeof module !== 'undefined') {
    factory(exports, module)
  } else {
    var mod = { exports: {} }
    factory(mod.exports, mod)
    global.CaijingStockJSBridge = mod.exports
  }
})(this, function(exports, module) {
  function setupWebViewJavascriptBridge(callback) {
    if (window.WebViewJavascriptBridge) {
      return callback(WebViewJavascriptBridge)
    }
    if (window.WVJBCallbacks) {
      return window.WVJBCallbacks.push(callback)
    }
    window.WVJBCallbacks = [callback]
    var WVJBIframe = document.createElement('iframe')
    WVJBIframe.style.display = 'none'
    WVJBIframe.src = 'https://__bridge_loaded__'
    document.documentElement.appendChild(WVJBIframe)
    setTimeout(function() {
      document.documentElement.removeChild(WVJBIframe)
    }, 0)
  }
  var once = function once(fn) {
    var called = false
    return function() {
      if (called) return
      called = true
      for (
        var _len = arguments.length, args = new Array(_len), _key = 0;
        _key < _len;
        _key++
      ) {
        args[_key] = arguments[_key]
      }
      return fn.apply(this, args)
    }
  }
  var proxyWindow = function proxyWindow() {
    var _stockJsbridgeAndroid = null
    Object.defineProperty(window, 'stockJsbridgeAndroid', {
      set: function set(newvalue) {
        if (!_stockJsbridgeAndroid) {
          _stockJsbridgeAndroid = newvalue
        }
      },
      get: function get() {
        return _stockJsbridgeAndroid
      }
    })
  }
  function Bridge() {
    this.isIos = !!navigator.userAgent.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/)
    this.callBuffer = []
    this.registerBuffer = []
    this.bridge = null
    this.init()
  }
  Bridge.prototype.init = function init() {
    var _this = this
    if (_this.isIos) {
      setupWebViewJavascriptBridge(function(bridge) {
        _this.bridge = bridge
        _this.runBuffers()
      })
    } else {
      proxyWindow()
      if (window.stockJsbridgeAndroid) {
        _this.bridge = window.stockJsbridgeAndroid
      } else {
        document.addEventListener(
          'onstockJsbridgeAndroidReady',
          once(function() {
            _this.bridge = window.stockJsbridgeAndroid
            _this.runBuffers()
          })
        )
      }
    }
  }
  Bridge.prototype.runBuffers = function runBuffers() {
    var _this = this
    if (_this.callBuffer.length > 0) {
      _this.callBuffer.forEach(function(args) {
        _this.bridge.callHandler(args[0], args[1], args[2])
      })
    }
    if (_this.registerBuffer.length > 0) {
      _this.registerBuffer.forEach(function(args) {
        _this.bridge.registerHandler(args[0], args[1])
      })
    }
  }
  Bridge.prototype.callHandler = function(p1, p2, p3) {
    if (!p1) {
      throw new Error('params need')
    }
    if (typeof p2 === 'function') {
      p3 = p2
      p2 = null
    }
    if (!p2) {
      p2 = {}
    }
    if (!p3) {
      p3 = function p3() {}
    }
    if (!this.bridge) {
      this.callBuffer.push([p1, p2, p3])
    } else {
      this.bridge.callHandler(p1, p2, p3)
    }
  }
  Bridge.prototype.registerHandler = function(p1, p2) {
    if (!p1 || !p2) {
      throw new Error('params need')
    }
    if (!this.bridge) {
      this.registerBuffer.push([p1, p2])
    } else {
      this.bridge.registerHandler(p1, p2)
    }
  }
  var _bridge = new Bridge()
  module.exports = _bridge
})
